#region License
/* Copyright 2004, 2013 James F. Bellinger
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redestribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must
 * not claim that you wrote the original software. If you use this software
 * in a product, an acknowledgement in the product documentation would be
 * appreciated but is not required.
 * 
 * 2. Altered source versions must be plainly marked as such, and must
 * not be misrepresented as being the original software.
 * 
 * 3. This notice may not be removed or altered from any source
 * distribution. */
#endregion

using System;

namespace Chatter.Plugins.Server
{
	/// <summary>
	/// Offers basic information about the chat client, commands to communicate
	/// with the chat server, and methods to query the user.
	/// </summary>
	public interface IChatController
	{
		string	ActiveChannel   { get; }
		string	ApplicationName { get; }
		bool    IsConnected     { get; }
		string	Nickname        { get; }
		Version Version         { get; }

		/// <summary>
		/// Joins the specified channel.
		/// </summary>
		/// <param name="channelName">The channel to join.</param>
		void Join(string channelName);

		/// <summary>
		/// Leaves the specified channel.
		/// </summary>
		/// <param name="channelName">The channel to leave.</param>
		void Leave(string channelName);

		/// <summary>
		/// Sends a message.
		/// </summary>
		/// <param name="recipient">The user or channel to send the message to.</param>
		/// <param name="messageType">The type of message to send.</param>
		/// <param name="messageText">The text of the message.</param>
		void Message(string recipient, MessageType messageType,
			string messageText);

		/// <summary>
		/// Opens a modal query dialog.
		/// </summary>
		/// <param name="queryMessage">The text of the query message.</param>
		/// <param name="queryTitle">The query dialog's title.</param>
		/// <param name="userResponse">Returns the user's response.</param>
		/// <returns>Returns true if the user clicked OK, or false if the
		/// user clicked Cancel.</returns>
		bool QueryUser(string queryMessage, string queryTitle,
			out string userResponse);
	}
}