#region License
/* Copyright 2004, 2013 James F. Bellinger
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redestribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must
 * not claim that you wrote the original software. If you use this software
 * in a product, an acknowledgement in the product documentation would be
 * appreciated but is not required.
 * 
 * 2. Altered source versions must be plainly marked as such, and must
 * not be misrepresented as being the original software.
 * 
 * 3. This notice may not be removed or altered from any source
 * distribution. */
#endregion

using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows.Forms;
using SecondLanguage;

namespace Chatter
{
	public class ChannelList : System.Windows.Forms.Form
	{
        static Translator _ = Translator.Default;

        private System.ComponentModel.Container components = null;
        private ListView lstChannels;
        private ColumnHeader columnHeader1;
        private ColumnHeader columnHeader2;
        private ColumnHeader columnHeader3;
		private ChatterMDI main;

        List<ChannelInfo> cache;
        int sortColumn;
        bool sortReversed;

		public ChannelList(ChatterMDI main)
		{
            InitializeComponent(); this.main = main; cache = new List<ChannelInfo>();

			lstChannels.DoubleClick += new EventHandler(lstChannels_DoubleClick);
            UpdateSorting(1);
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}

			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.lstChannels = new System.Windows.Forms.ListView();
            this.columnHeader1 = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));
            this.columnHeader2 = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));
            this.columnHeader3 = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));
            this.SuspendLayout();
            // 
            // lstChannels
            // 
            this.lstChannels.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.columnHeader1,
            this.columnHeader2,
            this.columnHeader3});
            this.lstChannels.Dock = System.Windows.Forms.DockStyle.Fill;
            this.lstChannels.FullRowSelect = true;
            this.lstChannels.Location = new System.Drawing.Point(0, 0);
            this.lstChannels.MultiSelect = false;
            this.lstChannels.Name = "lstChannels";
            this.lstChannels.Size = new System.Drawing.Size(573, 370);
            this.lstChannels.TabIndex = 0;
            this.lstChannels.UseCompatibleStateImageBehavior = false;
            this.lstChannels.View = System.Windows.Forms.View.Details;
            this.lstChannels.ColumnClick += new System.Windows.Forms.ColumnClickEventHandler(this.lstChannels_ColumnClick);
            // 
            // columnHeader1
            // 
            this.columnHeader1.Text = "Name";
            this.columnHeader1.Width = 100;
            // 
            // columnHeader2
            // 
            this.columnHeader2.Text = "Users";
            this.columnHeader2.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
            // 
            // columnHeader3
            // 
            this.columnHeader3.Text = "Description";
            this.columnHeader3.Width = 400;
            // 
            // ChannelList
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(573, 370);
            this.Controls.Add(this.lstChannels);
            this.Name = "ChannelList";
            this.Load += new System.EventHandler(this.ChannelList_Load);
            this.ResumeLayout(false);

		}
		#endregion

		private void lstChannels_DoubleClick(object sender, EventArgs e)
		{
			var items = lstChannels.SelectedItems;
            if (items.Count >= 1)
            {
                var item = items[0].Tag as ChannelInfo;
                if (item != null)
                {
                    main.JoinChannel(item.Name);
                }
            }
		}

		private void ChannelList_Load(object sender, System.EventArgs e)
		{
			this.lstChannels.Font = main.Fonts.Channels;
            UpdateTitle();
		}

		private class ChannelInfo
		{
			public string Name;
			public int    Users;
			public string Description;
		}

		public void ClearChannels()
		{
			lstChannels.Items.Clear(); cache.Clear();
		}

		public void Done()
		{
			if (cache.Count > 0)
			{
                lstChannels.Items.AddRange(cache.Select(
                    x =>
                        new ListViewItem(new string[] { x.Name, x.Users.ToString(), x.Description }) { Tag = x }
                    ).ToArray());
                cache.Clear();

                UpdateTitle();
			}
		}

        void UpdateTitle()
        {
            Text = _["Channel List - {0} channels", lstChannels.Items.Count];
        }

		public void AddChannel(string Name, int Users, string Description)
		{
			if (Name != "*")
			{
				ChannelInfo ci = new ChannelInfo();
				ci.Name = Name; ci.Users = Users; ci.Description = Description;

				cache.Add(ci); int maxCached = Math.Max(32, lstChannels.Items.Count / 8);
				if (cache.Count >= maxCached) { Done(); }
			}
		}

        sealed class SortComparer : IComparer
        {
            public int Column { get; set; }
            public bool Reversed { get; set; }

            public int Compare(object o1, object o2)
            {
                int c;
                var lv1 = (ListViewItem)o1;
                var lv2 = (ListViewItem)o2;
                var txt1 = lv1.SubItems[Column].Text;
                var txt2 = lv2.SubItems[Column].Text;

                if (Column == 1)
                {
                    // Intentionally reversed. When sorting by user count,
                    // generally one is looking for the populated channels.
                    c = -int.Parse(txt1).CompareTo(int.Parse(txt2));
                }
                else
                {
                    c = string.Compare(txt1, txt2);
                }

                if (Reversed) { c = -c; }
                return c;
            }
        }

        void UpdateSorting(int column)
        {
            sortReversed = column == sortColumn && !sortReversed;
            sortColumn = column;

            lstChannels.ListViewItemSorter = new SortComparer()
            {
                Column = column,
                Reversed = sortReversed
            };
        }

        private void lstChannels_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            UpdateSorting(e.Column);
        }
	}
}
